from gc import disable
from pickle import TRUE
from typing import Dict, Any

from InquirerPy import inquirer
from InquirerPy.base.control import Choice
from InquirerPy.separator import Separator


from rich.console import Console
from rich.panel import Panel
from rich.prompt import Prompt
from rich.text import Text
from pathlib import Path
import json
from .browsers import get_sessionid

CREDS_PATH: Path = Path.home() / ".maktabkhooneh_dl" / "session.json"
CREDS_PATH.parent.mkdir(parents=True, exist_ok=True)

console = Console()


def _load_json() -> Dict[str, Any]:
    """Load existing credentials, return empty dict if file doesn't exist."""
    if not CREDS_PATH.exists():
        return {}
    try:
        with CREDS_PATH.open("r", encoding="utf-8") as fp:
            return json.load(fp)
    except (json.JSONDecodeError, OSError):
        return {}


def _save_json(data: Dict[str, Any]) -> None:
    """Save dictionary to JSON file."""
    with CREDS_PATH.open("w", encoding="utf-8") as fp:
        json.dump(data, fp, indent=2, ensure_ascii=False)


def auto_login():
    confirm = inquirer.confirm(
        message="Continue since all browsers are closed?", default=True
    ).execute()

    session_id = get_sessionid()
    if not session_id:
        console.print(
            "[red]No browser with an active login was found. Try logging in manually.[/red]"
        )
        manual_login()
        return 2
    creds = _load_json()
    creds["session_id"] = session_id
    _save_json(creds)
    console.print(
        Panel(
            Text("Session id saved successfully!", style="bold green"),
            border_style="green",
        )
    )
    return 2


def manual_login():
    console.print(
        Panel(
            Text(
                "Enter the session id copied from your browser cookies "
                "(usually named `sessionid`).",
                style="cyan",
            ),
            title="Manual Login",
            border_style="bright_blue",
        )
    )

    session_id = Prompt.ask("Session id", console=console).strip()
    if not session_id:
        console.print("[red]Session id cannot be empty.[/red]")
        manual_login()
        return 2

    creds = _load_json()
    creds["session_id"] = session_id
    _save_json(creds)

    console.print(
        Panel(
            Text("Session id saved successfully!", style="bold green"),
            border_style="green",
        )
    )
    return 2


def check_login() -> str:
    """
    Returns the stored session_id if it exists, otherwise an empty string.
    """
    return _load_json().get("session_id", "")



from codecs import charmap_build
import time
from datetime import datetime, timedelta
from rich.live import Live
from pathlib import Path
from InquirerPy import base, inquirer
import requests
from bs4 import BeautifulSoup
from rich.console import Console
from rich.panel import Panel
from rich.prompt import Prompt
from rich.text import Text
import json
from rich.progress import (
    Progress,
    BarColumn,
    DownloadColumn,
    TextColumn,
    TransferSpeedColumn,
    TimeRemainingColumn,
)
import os

console = Console()
session = requests.Session()


def _check_course(url, sessionid):
    base_url = f"https://maktabkhooneh.org/api/v1/sale/{url}/prices"
    session.cookies.set("sessionid", sessionid)
    r = session.get(base_url)
    if r.status_code == 200:
        if not json.loads(r.text)["can_purchase"]["CONTENT"]:
            return True
    return False


def _get_course_chapters(url):
    base_url = f"https://maktabkhooneh.org/api/v1/courses/{url}/chapters/"
    r = session.get(base_url)
    if r.status_code == 200:
        data = json.loads(r.text)
        chapters = {}
        for chapter in data["chapters"]:
            chapters[f"{chapter["slug"]}-ch{chapter["id"]}"] = [
                unit["slug"]
                for unit in chapter["unit_set"]
                if unit["type"] == "lecture"
            ]

        return chapters


def _get_unit_links(url, chapters):
    base_url = f"https://maktabkhooneh.org/course/{url}"
    unit_links = []
    for chapter in chapters:
        for unit in chapters[chapter]:
            unit_links.append(f"{base_url}/{chapter}/{unit}")
    return unit_links


def _get_video_links(unit_links, high_quality=True):
    for unit in unit_links:
        r = session.get(unit)
        if r.status_code == 200:
            soup = BeautifulSoup(r.text, "html.parser")
            try:
                if high_quality:
                    yield soup.find_all(
                        "a",
                        attrs={
                            "onclick": lambda x: x and "send_download_event(1)" in x
                        },
                    )[0]["href"]
                else:
                    yield soup.find_all(
                        "a",
                        attrs={
                            "onclick": lambda x: x and "send_download_event(0)" in x
                        },
                    )[0]["href"]
            except IndexError:
                pass


def _download_video(url, filepath):
    """
    Download a video to the specified path with resume support.
    Assumes 'filepath' always includes directory (e.g., 'folder/video.mp4').
    """
    # Create directory if it doesn't exist
    os.makedirs(os.path.dirname(filepath), exist_ok=True)

    # Get file size from server
    r = requests.head(url)
    r.raise_for_status()
    total_size = int(r.headers.get("content-length", 0))
    if total_size == 0:
        raise ValueError(
            "Server did not return file size or does not support range requests."
        )

    # Resume logic
    initial_pos = os.path.getsize(filepath) if os.path.exists(filepath) else 0
    if initial_pos >= total_size:
        print(f"✅ {filepath} is already fully downloaded.")
        return

    # Setup progress bar
    progress = Progress(
        "[blue]{task.description}",
        BarColumn(),
        DownloadColumn(),
        TransferSpeedColumn(),
        TimeRemainingColumn(),
    )

    with progress:
        task = progress.add_task(
            f"Downloading {os.path.basename(filepath)}",
            total=total_size,
            completed=initial_pos,
        )
        headers = {"Range": f"bytes={initial_pos}-"} if initial_pos else {}

        with requests.get(url, headers=headers, stream=True) as r:
            r.raise_for_status()
            mode = "ab" if initial_pos > 0 else "wb"
            with open(filepath, mode) as f:
                for chunk in r.iter_content(1024 * 1024):  # 1 MB chunks
                    if chunk:
                        f.write(chunk)
                        progress.advance(task, len(chunk))

    print(f"✅ Download completed: {filepath}")


def _normalize_file_name(unit_links, i):
    file_name = unit_links[i].split("/")[-2:]
    file_name[0] = "-".join(file_name[0].split("-")[:-1])
    file_name[1:1] = ["/"]
    file_name.append(".mp4")
    file_name = "".join(file_name)
    return str(Path.cwd() / file_name)


def start_download(sessionid, course_url=""):
    if not course_url:
        course_url = Prompt.ask("course url", console=console).strip().split("/")[-1]
    if not _check_course(course_url, sessionid):
        console.print(
            Panel(
                Text("you have to buy the course first", style="bold red"),
                border_style="red",
            )
        )
        return 0
    chapters = _get_course_chapters(course_url)
    unit_links = _get_unit_links(course_url, chapters)

    for i, video_link in enumerate(_get_video_links(unit_links)):
        file_name = _normalize_file_name(unit_links, i)
        _download_video(video_link, file_name)

    return 1


def video_links_txt(sessionid):
    console = Console()
    course_url = Prompt.ask("course url", console=console).strip().split("/")[-1]

    if not _check_course(course_url, sessionid):
        console.print(
            Panel(
                Text("you have to buy the course first", style="bold red"),
                border_style="red",
            )
        )
        return 0

    chapters = _get_course_chapters(course_url)
    unit_links = _get_unit_links(course_url, chapters)
    total = len(unit_links)

    with open(str(Path.cwd() / f"{course_url}.txt"), "a", encoding="utf-8") as file:
        with Progress(
            TextColumn("[progress.description]{task.description}"),
            BarColumn(),
            TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
            TimeRemainingColumn(),
            TextColumn("[blue]{task.completed}/{task.total}"),
            console=console,
        ) as progress:
            task = progress.add_task("Processing...", total=total)
            for video_link in _get_video_links(unit_links):
                file.write(f"{video_link}\n")
                progress.update(task, advance=1)
                time.sleep(0.2)
    console.print(
        Panel(
            Text("✅ your file is valid until the next hour", style="bold yellow"),
            border_style="green",
        )
    )
    return 1


def schedule_download(sessionid):
    course_url = Prompt.ask("course url", console=console).strip().split("/")[-1]
    hours = [f"{i:02d}" for i in range(24)]
    minutes = [f"{i:02d}" for i in range(0, 60, 5)]

    console.print("🕐 [bold blue]Select your download time[/bold blue]")
    hour = inquirer.select(
        message="Choose hour:",
        choices=hours,
        default="00",
    ).execute()
    minute = inquirer.select(
        message="Choose minutes:",
        choices=minutes,
        default="00",
    ).execute()

    console.print(f"[bold green]Download scheduled for {hour}:{minute}[/bold green]")

    now = datetime.now()
    target_time = now.replace(
        hour=int(hour), minute=int(minute), second=0, microsecond=0
    )
    if target_time <= now:
        target_time += timedelta(days=1)

    with Live(console=console, refresh_per_second=1) as live:
        while True:
            now = datetime.now()

            if now >= target_time:
                break

            remaining = target_time - now
            total_seconds = int(remaining.total_seconds())
            hours_left, remainder = divmod(total_seconds, 3600)
            minutes_left, seconds_left = divmod(remainder, 60)

            text = Text()
            text.append("⏰ ", style="yellow bold")
            text.append("Time left: ", style="white bold")

            if hours_left > 0:
                text.append(f"{hours_left:02d}h ", style="cyan bold")
            if minutes_left > 0:
                text.append(f"{minutes_left:02d}m ", style="magenta bold")
            if hours_left == 0 and minutes_left == 0 and seconds_left > 0:
                text.append(f"{seconds_left:02d}s", style="magenta bold")

            live.update(text)

            if total_seconds > 60:
                sleep_duration = min(60, total_seconds - 60)
            else:
                sleep_duration = 1

    console.print(
        "[bold green]scheduled time reached! Starting download...[/bold green]"
    )
    return start_download(sessionid, course_url=course_url)



import typer

from rich.console import Console


from .menus import download_menu, login_menu
from .auth.login import check_login

console = Console()
app = typer.Typer()


def handle_login():
    sessionid = check_login().strip()
    if not sessionid:
        login_menu()
        handle_login()
    else:
        if download_menu(sessionid) == 2:
            handle_login()


@app.command()
def start():
    console.print("[bold cyan]maktabkhooneh downloader[/bold cyan]")
    handle_login()


if __name__ == "__main__":
    app()



import re
from InquirerPy import inquirer
from InquirerPy.separator import Separator
from InquirerPy.base.control import Choice
from click.utils import R
from .download.downloader import start_download, video_links_txt, schedule_download
from .auth.login import auto_login, manual_login
from maktabkhooneh_downloader.auth import login


def download_menu(sessionid):
    choice = inquirer.select(
        message=f"logged in as sessionid: {sessionid}",
        choices=[
            Choice(
                value=lambda: start_download(sessionid),
                name="start download 🚀",
            ),
            Choice(
                value=lambda: schedule_download(sessionid),
                name="schedule download ⏰",
            ),
            Choice(
                value=lambda: video_links_txt(sessionid),
                name="get txt file of video links (for IDM) 📥",
            ),
            Choice(
                value=login_menu,
                name="change (login again) 🔄",
            ),
            Choice(value=exit, name="🚪 Exit"),
        ],
    ).execute()
    return choice()


def login_menu():
    choice = inquirer.select(
        message="Select an option for login:",
        choices=[
            Choice(
                value=auto_login,
                name="🔥 Auto login (from your browser cookies)",
            ),
            Choice(
                value=manual_login,
                name="⚡ Manual login (enter your session id manually)",
            ),
            Separator(),
            Choice(value=exit, name="🚪 Exit"),
        ],
    ).execute()
    return choice()



import os
import platform
import glob
from pathlib import Path
import browser_cookie3
from InquirerPy.base.control import Choice
from InquirerPy import inquirer


def _find_browser_paths():
    system = platform.system()
    home = Path.home()
    paths = {}

    if system == "Windows":
        paths = {
            "chrome": [
                home / "AppData/Local/Google/Chrome/User Data",
                home / "AppData/Local/Chromium/User Data",
            ],
            "firefox": [home / "AppData/Roaming/Mozilla/Firefox/Profiles"],
            "edge": [home / "AppData/Local/Microsoft/Edge/User Data"],
            "opera": [
                home / "AppData/Roaming/Opera Software/Opera Stable",
                home / "AppData/Local/Opera Software/Opera Stable",
            ],
            "opera_gx": [
                home / "AppData/Roaming/Opera Software/Opera GX Stable",
                home / "AppData/Local/Opera Software/Opera GX Stable",
            ],
            "brave": [home / "AppData/Local/BraveSoftware/Brave-Browser/User Data"],
            "vivaldi": [home / "AppData/Local/Vivaldi/User Data"],
        }
    elif system == "Darwin":  # macOS
        paths = {
            "chrome": [
                home / "Library/Application Support/Google/Chrome",
                home / "Library/Application Support/Chromium",
            ],
            "firefox": [home / "Library/Application Support/Firefox/Profiles"],
            "safari": [home / "Library/Cookies"],
            "edge": [home / "Library/Application Support/Microsoft Edge"],
            "opera": [home / "Library/Application Support/com.operasoftware.Opera"],
            "opera_gx": [
                home / "Library/Application Support/com.operasoftware.OperaGX"
            ],
            "brave": [home / "Library/Application Support/BraveSoftware/Brave-Browser"],
            "vivaldi": [home / "Library/Application Support/Vivaldi"],
        }
    else:  # Linux
        paths = {
            "chrome": [home / ".config/google-chrome", home / ".config/chromium"],
            "firefox": [home / ".mozilla/firefox"],
            "opera": [home / ".config/opera"],
            "opera_gx": [home / ".config/opera-gx"],
            "brave": [home / ".config/BraveSoftware/Brave-Browser"],
            "vivaldi": [home / ".config/vivaldi"],
        }

    found_browsers = {}
    for browser, browser_paths in paths.items():
        for path in browser_paths:
            if path.exists():
                found_browsers[browser] = path
                break
    return found_browsers


def _find_cookie_files(browser_path, browser_name):
    cookie_files = []

    if browser_name in [
        "chrome",
        "edge",
        "opera",
        "opera_gx",
        "brave",
        "vivaldi",
        "chromium",
    ]:
        patterns = [
            browser_path / "Default/Cookies",
            browser_path / "Default/Network/Cookies",
            browser_path / "Profile*/Cookies",
            browser_path / "Profile*/Network/Cookies",
        ]
        for pattern in patterns:
            if pattern.exists():
                cookie_files.append(pattern)
            else:
                cookie_files.extend(Path(f) for f in glob.glob(str(pattern)))

    elif browser_name == "firefox":
        for profile_dir in glob.glob(str(browser_path / "*.default*")):
            cookie_db = Path(profile_dir) / "cookies.sqlite"
            if cookie_db.exists():
                cookie_files.append(cookie_db)

    elif browser_name == "safari":
        cookie_file = browser_path / "Cookies.binarycookies"
        if cookie_file.exists():
            cookie_files.append(cookie_file)

    return cookie_files


def _extract_sessionid_cookie(cookie_file, browser_name):
    domain = "maktabkhooneh.org"
    cookie_extractors = {
        "chrome": browser_cookie3.chrome,
        "chromium": browser_cookie3.chrome,
        "firefox": browser_cookie3.firefox,
        "edge": browser_cookie3.edge,
        "opera": browser_cookie3.opera,
        "opera_gx": browser_cookie3.opera_gx,
        "brave": browser_cookie3.brave,
        "vivaldi": browser_cookie3.vivaldi,
        "safari": browser_cookie3.safari,
    }

    # Try extracting from specific file
    extractor = cookie_extractors.get(browser_name, browser_cookie3.chrome)
    try:
        cj = extractor(cookie_file=str(cookie_file), domain_name=domain)
        for cookie in cj:
            if cookie.name == "sessionid" and domain in cookie.domain:
                return cookie.value
    except Exception:
        pass

    # Fallback: try without specifying file
    try:
        cj = extractor(domain_name=domain)
        for cookie in cj:
            if cookie.name == "sessionid" and domain in cookie.domain:
                return cookie.value
    except Exception:
        pass

    # Final fallback to chrome if all else fails
    try:
        cj = browser_cookie3.chrome(domain_name=domain)
        for cookie in cj:
            if cookie.name == "sessionid" and domain in cookie.domain:
                return cookie.value
    except Exception:
        pass

    return None


def _get_sessionid_cookies():
    result = {}
    browsers = _find_browser_paths()

    for browser_name, browser_path in browsers.items():
        cookie_files = _find_cookie_files(browser_path, browser_name)
        for cookie_file in cookie_files:
            sessionid = _extract_sessionid_cookie(cookie_file, browser_name)
            if sessionid:
                result[browser_name] = sessionid
                break

    return result


def get_sessionid():
    sessionids = _get_sessionid_cookies()
    if len(sessionids) > 1:
        choices = []
        for key, value in sessionids.items():
            choices.append(Choice(value=value, name=key))

        choice = inquirer.select(
            message="Select a browser for login:", choices=choices
        ).execute()
        return choice
    return list(sessionids.values())[0]
